#include "global.h"
#include "iax.h"
#include "log.h"

static const char rcsid[] = "iaxproxy-version: " __FILE__ " " VCS_VERSION;

static const char *iax_frame_names[] = {
	"",
	"NEW",
	"PING",
	"PONG",
	"ACK",
	"HANGUP",
	"REJECT",
	"ACCEPT",
	"AUTHREQ",
	"AUTHREP",
	"INVAL",
	"LAGRQ",
	"LAGRP",
	"REGREQ",
	"REGAUTH",
	"REGACK",
	"REGREJ",
	"REGREL",
	"VNAK",
	"DPREQ",
	"DPREP",
	"DIAL",
	"TXREQ",
	"TXCNT",
	"TXACC",
	"TXREADY",
	"TXREL",
	"TXREJ",
	"QUELCH",
	"UNQUELCH",
	"POKE",
	"Reserved",
	"MWI",
	"UNSUPPORT",
	"TRANSFER",
	"PROVISION",
	"FWDOWNL",
	"FWDATA",
};

static const char *control_frame_names[] = {
	"",
	"Hangup",
	"Reserved",
	"Ringing",
	"Answer",
	"Busy",
	"Reserved",
	"Reserved",
	"Congestion",
	"Flash Hook",
	"Reserved",
	"Option",
	"Key Radio",
	"Unkey Radio",
	"Call Progress",
	"Call",
	"Hold",
	"Unhold",
};

// helper - get friendly name for type 6 packet
const char *iax_frame_name_get(unsigned int i) {
	if(i<=0x25)
		return iax_frame_names[i];
	else
		return iax_frame_names[0];
}

// helper - get friendly name for type 4 packet
const char *control_frame_name_get(unsigned int i) {
	if(i<=0x11)
		return control_frame_names[i];
	else
		return control_frame_names[0];
}

ie_position find_ie(struct iax_packet *frame, int len, ie_type ie_query) {
	ie_position pos = {NULL, 0};

	unsigned char *ptr, *ptr_end;
	unsigned char ie_data_len;

	ptr = &(frame->iedata[0]);
	ptr_end = ((unsigned char *)frame) + len;

	// keep inside the bounds of the packet
	while (ptr+1 < ptr_end) {
		ie_data_len = ptr[1];

		if (ptr[0] == ie_query) {
			// found the right IE
			if (ptr + ie_data_len + 2 <= ptr_end) {
				pos.start = ptr+2;
				pos.length = ie_data_len;
				return pos;
			} else {
				// pos should be initialised to NULL
				return pos;
			}
		}
		ptr += ie_data_len + 2;
	}

	return pos;
}

int iax_is_fullframe(struct iax_packet *frame) {
	return (ntohs(frame->scallno) & 0x8000) != 0;
}

int iax_is_retransmission(struct iax_packet *frame) {
	return (ntohs(frame->dcallno) & 0x8000) != 0;
}

void iax_set_retransmission(struct iax_packet *frame, int ret) {
	uint16_t callno = ntohs(frame->dcallno);
	if (ret)
		frame->dcallno = htons(0x8000 | callno);
	else
		frame->dcallno = htons(0x7fff & callno);
}

uint16_t iax_get_scallno(struct iax_packet* frame) {
	return ntohs(frame->scallno) & 0x7fff;
}
uint16_t iax_get_dcallno(struct iax_packet* frame) {
	return ntohs(frame->dcallno) & 0x7fff;
}

void log_packet(int level, struct iax_packet *frame) {
	LOG(level, "scallno:%i dcallno:%i ts:%i oseqno:%i iseqno:%i type:%s", ntohs(frame->scallno), ntohs(frame->dcallno), ntohl(frame->ts), frame->oseqno, frame->iseqno, FRAME_NAME(frame));
}

