#ifndef _IAX_H
#define _IAX_H 1

#include <sys/types.h>

#include "global.h"

// some useful constants from asterisk
#define IAX_TYPE_NEW	0x01
#define IAX_TYPE_PING	0x02
#define IAX_TYPE_PONG	0x03
#define IAX_TYPE_ACK	0x04
#define IAX_TYPE_HANGUP	0x05
#define IAX_TYPE_REJECT	0x06
#define IAX_TYPE_ACCEPT	0x07
#define IAX_TYPE_AUTHREQ	0x08
#define IAX_TYPE_AUTHREP	0x09
#define IAX_TYPE_INVAL	0x0a
#define IAX_TYPE_LAGRQ	0x0b
#define IAX_TYPE_LAGRP	0x0c
#define IAX_TYPE_REGREQ	0x0d
#define IAX_TYPE_REGAUTH	0x0e
#define IAX_TYPE_REGACK	0x0f
#define IAX_TYPE_REGREJ	0x10
#define IAX_TYPE_REGREL	0x11
#define IAX_TYPE_VNAK	0x12
#define IAX_TYPE_DPREQ	0x13
#define IAX_TYPE_DPREP	0x14
#define IAX_TYPE_DIAL	0x15
#define IAX_TYPE_TXREQ	0x16
#define IAX_TYPE_TXCNT	0x17
#define IAX_TYPE_TXACC	0x18
#define IAX_TYPE_TXREADY	0x19
#define IAX_TYPE_TXREL	0x1a
#define IAX_TYPE_TXREJ	0x1b
#define IAX_TYPE_QUELCH	0x1c
#define IAX_TYPE_UNQUELCH	0x1d
#define IAX_TYPE_POKE	0x1e
#define IAX_TYPE_MWI	0x20
#define IAX_TYPE_UNSUPPORT	0x21
#define IAX_TYPE_TRANSFER	0x22

#define AST_FRAME_IAX		6

// gets frame name - ACK, NEW, etc. - both IAX and control
#define FRAME_NAME(f) ((f)->type==6?iax_frame_name_get((f)->csub):((f)->type==4?control_frame_name_get((f)->csub):"<UNKNOWN>"))

typedef enum {
	IE_CALLED_NUMBER   = 0x01,
	IE_CALLING_NUMBER  = 0x02,
	IE_CALLING_ANI     = 0x03,
	IE_CALLING_NAME    = 0x04,
	IE_CALLED_CONTEXT  = 0x05,
	IE_USERNAME        = 0x06,
	IE_PASSWORD        = 0x07,
	IE_CAPABILITY      = 0x08,
	IE_FORMAT          = 0x09,
	IE_LANGUAGE        = 0x0a,
	IE_VERSION         = 0x0b,
	IE_ADSICPE         = 0x0c,
	IE_DNID            = 0x0d,
	IE_AUTHMETHODS     = 0x0e,
	IE_CHALLENGE       = 0x0f,
	IE_MD5_RESULT      = 0x10,
	IE_RSA_RESULT      = 0x11,
	IE_APPARENT_ADDR   = 0x12,
	IE_REFRESH         = 0x13,
	IE_DPSTATUS        = 0x14,
	IE_CALLNO          = 0x15,
	IE_CAUSE           = 0x16,
	IE_IAX_UNKNOWN     = 0x17,
	IE_MSGCOUNT        = 0x18,
	IE_AUTOANSWER      = 0x19,
	IE_MUSICONHOLD     = 0x1a,
	IE_TRANSFERID      = 0x1b,
	IE_RDNIS           = 0x1c,
	IE_DATETIME        = 0x1f,
	IE_CALLINGPRES     = 0x26,
	IE_CALLINGTON      = 0x27,
	IE_CALLINGTNS      = 0x28,
	IE_SAMPLINGRATE    = 0x29,
	IE_CAUSECODE       = 0x2a,
	IE_ENCRYPTION      = 0x2b,
	IE_ENCKEY          = 0x2c,
	IE_CODEC_PREFS     = 0x2d,
	IE_RR_JITTER       = 0x2e,
	IE_RR_LOSS         = 0x2f,
	IE_RR_PKTS         = 0x30,
	IE_RR_DELAY        = 0x31,
	IE_RR_DROPPED      = 0x32,
	IE_RR_OOO          = 0x33,
	IE_OSPTOKEN        = 0x34,
} ie_type;

typedef struct {
	unsigned char *start;
	size_t length;
} ie_position;

struct iax_packet {
	uint16_t scallno;	/* Source call number -- high bit must be 1 */
	uint16_t dcallno;	/* Destination call number -- high bit is 1 if retransmission */
	uint32_t ts;	/* 32-bit timestamp in milliseconds (from 1st transmission) */
	uint8_t oseqno;	/* Packet number (outgoing) */
	uint8_t iseqno;	/* Packet number (next incoming expected) */
	uint8_t type;		/* Frame type */
	uint8_t csub;	/* Compressed subclass */
	unsigned char iedata[0];
} __attribute__ ((__packed__));

struct iax_packet_mini {
	uint16_t scallno;	/* Source call number -- high bit must be 0 */
	uint8_t ts;
	unsigned char data[0];
} __attribute__((__packed__));

int iax_is_fullframe(struct iax_packet*);
int iax_is_retransmission(struct iax_packet*);
void iax_set_retransmission(struct iax_packet*, int ret);
uint16_t iax_get_scallno(struct iax_packet*);
uint16_t iax_get_dcallno(struct iax_packet*);

const char *iax_frame_name_get(unsigned int);
const char *control_frame_name_get(unsigned int);
ie_position find_ie(struct iax_packet *frame, int len, ie_type ie_query);
void log_packet(int level, struct iax_packet *frame);

#endif
