/*
IAX balancing proxy
Authors: Ben Smithrust, Stanislaw Pitucha

Copyright (c) 2007 Gradwell dot com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef _LIST_H
#define	_LIST_H

#include <stdlib.h>

#include "global.h"

typedef struct list_ptrs {
	/*@null@*/ struct list_ptrs *prev, *next;
} list_ptrs;

#define LIST_HEAD {NULL,NULL}

#define list_entry(ptr, type, member) \
	((type *)((char *)(ptr)-(unsigned long)(&((type *)0)->member)))

#define list_init(lm) do { (lm)->prev=NULL; (lm)->next=NULL; } while(0)

#define list_foreach(_head, _iter) for(_iter = (_head)->next; _iter!=NULL; _iter=_iter->next)
#define list_foreach_safe(_head, _next, _iter) for(_iter = (_head)->next, _next = (_iter)?(_iter->next):NULL; _iter!=NULL; _iter = _next, _next = (_iter)?(_iter->next):NULL)

static inline void list_add_after(struct list_ptrs* list, /*@out@*/ struct list_ptrs* item) {
	item->next=list->next;
	item->prev=list;
	list->next=item;
	if(item->next)
		item->next->prev=item;
}

static inline void list_remove(struct list_ptrs* item) {
	if(item->next)
		item->next->prev=item->prev;
	if(item->prev)
		item->prev->next=item->next;
	item->next=NULL;
	item->prev=NULL;
}

static inline int list_is_linked(struct list_ptrs* item) {
	return (item->next != NULL) || (item->prev != NULL);
}

#endif	/* _LIST_H */
