#ifndef _LOG_H
#define _LOG_H

#define ERROR 0
#define INFO 0
#define WARNING 1
#define NOTICE 2
#define DEBUG 3
#define VERBOSE_DEBUG 4

#include <err.h>
#include <errno.h>
#include <stdio.h>
#include <time.h>

char *time_string(time_t);

// macro stringify hack
#define STRINGIFY(X) #X
#define TOSTRING(X) STRINGIFY(X)
// ex. "file.c:123[DEBUG]: "
#define AT(level) "%s " __FILE__ ":" TOSTRING(__LINE__) "[" level "]: "

#define LOG_FILE (log_file?log_file:stderr)
#define CONNLOG_FILE (connlog_file?connlog_file:stderr)

#define LOG(level, format, ...) if(debug_level>=level) { fprintf(LOG_FILE, AT(#level) format "\n", time_string(0), ## __VA_ARGS__); }
#define DIE(format, ...) do { LOG(ERROR, format, ## __VA_ARGS__); fprintf(stderr, format, ## __VA_ARGS__); exit(EXIT_FAILURE); } while (0)
#define LOG_CONN(level, format, conn, ...) if(debug_level>=level) { fprintf(LOG_FILE, AT(#level) format ": ", time_string(0), ## __VA_ARGS__); connection_print(conn); }
// if(printf...) should always be >0 - using double if, to skip ";"
#define LOG_DO(level) if(debug_level>=level) if(fprintf(LOG_FILE, AT(#level) "\n", time_string(0)))
#define CLOG(format, ...) if(connlog_file) { fprintf(CONNLOG_FILE, "%s " format "\n", time_string(0), ## __VA_ARGS__); }

#ifdef DO_CRASH
#define CRASH(format, ...) do { LOG(ERROR, format, ## __VA_ARGS__); fprintf(stderr, format, ## __VA_ARGS__); abort(); } while (0)
#else
#define CRASH(format, ...) do { LOG(ERROR, "FATAL ERROR:"); LOG(ERROR, format, ## __VA_ARGS__); } while (0)
#endif

void open_logs(int daemonize);

extern FILE *log_file;
extern FILE *connlog_file;

extern int debug_level;
extern const char *log_filename, *connlog_filename;

#endif
