/*
IAX balancing proxy
Authors: Ben Smithrust, Stanislaw Pitucha

Copyright (c) 2007 Gradwell dot com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef _SERVERS_H
#define	_SERVERS_H

#include <netinet/in.h>

typedef struct server server;

#include "connections.h"
#include "global.h"
#include "list.h"

struct server {
	struct list_ptrs list; // list of servers
	struct sockaddr_in addr;
	int load;
	int enabled:1;
	int not_responsive:1;
	time_t last_activity, last_poke, last_pong;
	list_ptrs opening_conns_head;
	connection *connections[32768]; // connections by server callno
	char *ip_string;
};

extern const char *servers_list_filename;

int server_enable_ip(in_addr_t addr);
int server_enable(const char* ip);
int server_disable_ip(in_addr_t addr);
int server_disable(const char* ip);
/*@null@*/ server *server_get_best(void);
void server_list_reload(void);
/*@null@*/ server *server_get(in_addr_t addr);
void server_destroy_all(void);

extern struct list_ptrs servers;

#endif	/* _SERVERS_H */

