#include <stdlib.h>
#include <check.h>
#include "../src/iax.h"

START_TEST(test_iefind_not_found)
{
	struct iax_packet frame_full = {
		.dcallno=0, .ts=0,
		.oseqno=0, .iseqno=0,
		.type=AST_FRAME_IAX, .csub=IAX_TYPE_POKE
	};

	ie_position iep = find_ie(&frame_full, sizeof(struct iax_packet), IE_CALLED_NUMBER);

	fail_unless(iep.start == NULL, "IE found, but should not be present");
}
END_TEST

START_TEST(test_iefind_empty)
{
	char frame_full_raw[sizeof(struct iax_packet)+2] = {0};
	struct iax_packet *frame_full = (struct iax_packet*) &frame_full_raw;
	frame_full->iedata[0] = IE_CALLED_NUMBER;

	ie_position iep = find_ie(frame_full, sizeof(struct iax_packet)+2, IE_CALLED_NUMBER);

	fail_unless((char*)iep.start == ((char*)&frame_full->iedata)+2, "IE address is wrong");
	ck_assert_int_eq(iep.length, 0);
}
END_TEST

START_TEST(test_iefind_4byte)
{
	char frame_full_raw[sizeof(struct iax_packet)+2+4] = {0};
	struct iax_packet *frame_full = (struct iax_packet*) &frame_full_raw;
	frame_full->iedata[0] = IE_CALLED_NUMBER;
	frame_full->iedata[1] = 4;

	ie_position iep = find_ie(frame_full, sizeof(struct iax_packet)+2+4, IE_CALLED_NUMBER);

	fail_unless((char*)iep.start == ((char*)&frame_full->iedata)+2, "IE address is wrong");
	ck_assert_int_eq(iep.length, 4);
}
END_TEST

START_TEST(test_iefind_tooshort)
{
	char frame_full_raw[sizeof(struct iax_packet)+2+1] = {0};
	struct iax_packet *frame_full = (struct iax_packet*) &frame_full_raw;
	frame_full->iedata[0] = IE_CALLED_NUMBER;
	frame_full->iedata[1] = 0;
	frame_full->iedata[2] = IE_CALLING_NUMBER;

	ie_position iep = find_ie(frame_full, sizeof(struct iax_packet)+2+1, IE_CALLED_NUMBER);

	fail_unless((char*)iep.start == ((char*)&frame_full->iedata)+2, "IE address is wrong");
	
	iep = find_ie(frame_full, sizeof(struct iax_packet)+2+1, IE_CALLING_NUMBER);
	
	fail_unless(iep.start == NULL, "IE found, but should not be present");
}
END_TEST

START_TEST(test_iefind_toolong)
{
	char frame_full_raw[sizeof(struct iax_packet)+2+10] = {0};
	struct iax_packet *frame_full = (struct iax_packet*) &frame_full_raw;
	frame_full->iedata[0] = IE_CALLED_NUMBER;
	frame_full->iedata[1] = 100;

	ie_position iep = find_ie(frame_full, sizeof(struct iax_packet)+2+10, IE_CALLED_NUMBER);
	fail_unless(iep.start == NULL, "IE found, but should not be present");
}
END_TEST

Suite *iaxproxy_suite (void) {
	Suite *s = suite_create ("Iaxproxy");

	/* Core test case */
	TCase *tc_core = tcase_create ("iefind");
	tcase_add_test (tc_core, test_iefind_not_found);
	tcase_add_test (tc_core, test_iefind_empty);
	tcase_add_test (tc_core, test_iefind_4byte);
	tcase_add_test (tc_core, test_iefind_tooshort);
	tcase_add_test (tc_core, test_iefind_toolong);
	suite_add_tcase (s, tc_core);

	return s;
}
		
int main (void) {
	int number_failed;
	Suite *s = iaxproxy_suite ();
	SRunner *sr = srunner_create (s);
	srunner_run_all (sr, CK_NORMAL);
	number_failed = srunner_ntests_failed (sr);
	srunner_free (sr);
	return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}

